@@ -1,10 +1,11 @@
 from . import io
+from .acquisition import *
 from .beams import *
 from .calibration import *
 from .cl import *
 from .instrument import *
-from .acquisition import *
 from .mapmaking import *
 from .pointings import *
+from .scene import *
 
-__version__ = '4.0'
+__version__ = '4.0.1'
