@@ -7,9 +7,14 @@ except:
     pass
 import numexpr as ne
 import numpy as np
-from pyoperators import Spherical2CartesianOperator, MPI
+from pyoperators import (
+    DenseBlockDiagonalOperator, IdentityOperator, HomothetyOperator,
+    ReshapeOperator, Rotation2dOperator, Rotation3dOperator,
+    Spherical2CartesianOperator)
 from pyoperators.utils import strenum
-from pysimulators import Instrument, LayoutVertex, ProjectionOperator
+from pysimulators import (
+    Instrument, LayoutVertex, ConvolutionTruncatedExponentialOperator,
+    ProjectionOperator)
 from pysimulators.interfaces.healpy import Cartesian2HealpixOperator
 from pysimulators.sparse import (
     FSRMatrix, FSRRotation2dMatrix, FSRRotation3dMatrix)
@@ -31,7 +36,8 @@ class QubicInstrument(Instrument):
     def __init__(self, name=None, calibration=None,
                  detector_sigma=10, detector_fknee=0, detector_fslope=1,
                  detector_ncorr=10, detector_tau=0.01,
-                 synthbeam_fraction=0.99, ngrids=2, nside=256, **keywords):
+                 synthbeam_dtype=np.float32, synthbeam_fraction=0.99,
+                 ngrids=2, nside=256, **keywords):
         """
         Parameters
         ----------
@@ -51,16 +57,14 @@ class QubicInstrument(Instrument):
             The detector 1/f correlation length.
         nside : int, optional
             Deprecated.
+        synthbeam_dtype : dtype, optional
+            The data type for the synthetic beams (default: float32).
+            It is the dtype used to store the values of the pointing matrix.
         synthbeam_fraction: float, optional
             The fraction of significant peaks retained for the computation
             of the synthetic beam.
         ngrids : int, optional
             Number of detector grids.
-        nu : float, optional
-            The operating monochromatic frequency or the filter central
-            frequency, in Hz.
-        dnu_nu : float, optional
-            The filter width.
 
         """
         if name is not None:
@@ -87,7 +91,7 @@ class QubicInstrument(Instrument):
         self._init_primary_beam()
         self._init_optics(**keywords)
         self._init_horns()
-        self._init_synthetic_beam(synthbeam_fraction)
+        self._init_synthetic_beam(synthbeam_dtype, synthbeam_fraction)
 
     def _get_detector_layout(self, ngrids, sigma, fknee, fslope, ncorr,
                              tau):
@@ -135,10 +139,11 @@ class QubicInstrument(Instrument):
     def _init_horns(self):
         self.horn = self.calibration.get('hornarray')
 
-    def _init_synthetic_beam(self, synthbeam_fraction):
+    def _init_synthetic_beam(self, synthbeam_dtype, synthbeam_fraction):
         class SyntheticBeam(object):
             pass
         sb = SyntheticBeam()
+        sb.dtype = np.dtype(synthbeam_dtype)
         sb.fraction = synthbeam_fraction
         self.synthetic_beam = sb
 
@@ -166,37 +171,95 @@ class QubicInstrument(Instrument):
             mp.autoscale()
         mp.show()
 
-    def get_projection_operator(self, scene, rotation, dtype=None,
-                                synthbeam_fraction=None, verbose=True):
+    def get_noise(self, sampling, out=None):
+        """
+        Return a noisy timeline.
+
+        """
+        return Instrument.get_noise(
+            self, sampling, sigma=self.detector.sigma,
+            fknee=self.detector.fknee, fslope=self.detector.fslope, out=out)
+
+    def get_detector_response_operator(self, sampling, tau=None):
+        """
+        Return the operator for the bolometer responses.
+
+        """
+        if tau is None:
+            tau = self.detector.tau
+        sampling_period = sampling.period
+        shapein = len(self), len(sampling)
+        if sampling_period == 0:
+            return IdentityOperator(shapein)
+        return ConvolutionTruncatedExponentialOperator(
+            tau / sampling_period, shapein=shapein)
+
+    def get_hwp_operator(self, sampling, scene):
+        """
+        Return the rotation matrix for the half-wave plate.
+
+        """
+        shape = (len(self), len(sampling))
+        if scene.kind == 'I':
+            return IdentityOperator(shapein=shape)
+        if scene.kind == 'QU':
+            return Rotation2dOperator(-4 * sampling.angle_hwp,
+                                      degrees=True, shapein=shape + (2,))
+        return Rotation3dOperator('X', -4 * sampling.angle_hwp,
+                                  degrees=True, shapein=shape + (3,))
+
+    def get_invntt_operator(self, sampling):
+        """
+        Return the inverse time-time noise correlation matrix as an Operator.
+
+        """
+        return Instrument.get_invntt_operator(
+            self, sampling, sigma=self.detector.sigma,
+            fknee=self.detector.fknee, fslope=self.detector.fslope,
+            ncorr=self.detector.ncorr)
+
+    def get_polarizer_operator(self, sampling, scene):
+        """
+        Return operator for the polarizer grid.
+
+        """
+        if scene.kind == 'I':
+            return HomothetyOperator(1 / self.detector.ngrids)
+
+        if self.detector.ngrids == 1:
+            raise ValueError(
+                'Polarized input not handled by a single detector grid.')
+
+        nd = len(self)
+        nt = len(sampling)
+        grid = self.detector.quadrant // 4
+        z = np.zeros(nd)
+        data = np.array([z + 0.5, 0.5 - grid, z]).T[:, None, None, :]
+        return ReshapeOperator((nd, nt, 1), (nd, nt)) * \
+            DenseBlockDiagonalOperator(data, shapein=(nd, nt, 3))
+
+    def get_projection_operator(self, sampling, scene, verbose=True):
         """
         Return the peak sampling operator.
 
         Parameters
         ----------
-        rotation : ndarray (ntimes, 3, 3)
-            The Reference-to-Instrument rotation matrix.
+        sampling : QubicSampling
+            The pointing information.
         scene : QubicScene
             The observed scene.
-        dtype : dtype
-            The datatype of the elements in the projection matrix.
-        synthbeam_fraction : float, optional
-            Override the instrument synthetic beam fraction.
+        verbose : bool, optional
+            If true, display information about the memory allocation.
 
         """
-        if synthbeam_fraction is None:
-            synthbeam_fraction = self.synthetic_beam.fraction
-        if dtype is None:
-            dtype = np.float32
-        dtype = np.dtype(dtype)
+        rotation = sampling.cartesian_galactic2instrument
+        dtype = self.synthetic_beam.dtype
+        fraction = self.synthetic_beam.fraction
         ndetectors = len(self)
-        if rotation.data.ndim == 2:
-            ntimes = 1
-        else:
-            ntimes = rotation.data.shape[0]
+        ntimes = len(sampling)
         nside = scene.nside
 
-        theta, phi, vals = _peak_angles_fraction(self, scene,
-                                                 synthbeam_fraction)
+        theta, phi, vals = _peak_angles_fraction(self, scene, fraction)
         ncolmax = theta.shape[-1]
         thetaphi = _pack_vector(theta, phi)  # (ndetectors, ncolmax, 2)
         direction = Spherical2CartesianOperator('zenith,azimuth')(thetaphi)
