@@ -12,74 +12,40 @@ from .utils import progress_bar
 __all__ = ['map2tod', 'tod2map_all', 'tod2map_each']
 
 
-def _get_operator(acq, max_nbytes=None, verbose=True):
-    projection = _get_projection(acq, max_nbytes=max_nbytes, verbose=verbose)
-    hwp = _get_hwp(acq, max_nbytes=max_nbytes)
-    polarizer = acq.get_polarizer_operator()
-    response = acq.get_detector_response_operator()
-
-    with rule_manager(inplace=True):
-        H = response * polarizer * (hwp * projection)
-    if acq.scene.kind == 'QU':
-        H = acq.get_subtract_grids_operator() * H
-
-    return H
-
-
-def _get_projection(acq, max_nbytes=None, verbose=True):
-    nbytes = acq.get_projection_nbytes()
-    if max_nbytes is None or nbytes <= max_nbytes:
-        P = acq.get_projection_operator(verbose=verbose)
-    else:
-        n = int(np.ceil(nbytes / max_nbytes))
-        samplings = acq.sampling.split(n)
-        acqs = [QubicAcquisition(acq.instrument, _, acq.scene)
-                for _ in samplings]
-
-        def callback(i):
-            return acqs[i].get_projection_operator(verbose=False)
-        P = BlockColumnOperator(proxy_group(n, callback), axisout=1)
-    return _distribute(acq, P)
-
-
-def _get_projection_restricted(acq, P, mask, max_nbytes=None, verbose=True):
-    nbytes = acq.get_projection_nbytes()
-    if max_nbytes is None or nbytes <= max_nbytes:
-        if acq.comm.size > 1:
-            P = P.operands[0]
+def _get_projection_restricted(acq, P, mask):
+    #XXX HACK
+    if len(acq.block) == 1:
         P.restrict(mask)
-    else:
-        n = int(np.ceil(nbytes / max_nbytes))
-        samplings = acq.sampling.split(n)
-        acqs = [QubicAcquisition(acq.instrument, _, acq.scene)
-                for _ in samplings]
-
-        def callback(i):
-            p = acqs[i].get_projection_operator(verbose=False)
-            p.restrict(mask)
-            return p
-        P = BlockColumnOperator(proxy_group(n, callback), axisout=1)
-    return _distribute(acq, P)
-
-
-def _get_hwp(acq, max_nbytes=None):
-    nbytes = acq.get_projection_nbytes()
-    if max_nbytes is None or nbytes <= max_nbytes:
-        return acq.get_hwp_operator()
-
-    n = int(np.ceil(nbytes / max_nbytes))
-    samplings = acq.sampling.split(n)
-    acqs = [QubicAcquisition(acq.instrument, _) for _ in samplings]
+        return P
 
     def callback(i):
-        return acqs[i].get_hwp_operator()
-    return BlockDiagonalOperator(proxy_group(n, callback), axisout=1)
+        f = acq.instrument.get_projection_operator
+        p = f(acq.sampling[acq.block[i]], acq.scene, verbose=False)
+        p.restrict(mask)
+        return p
+    return BlockColumnOperator(proxy_group(len(acq.block), callback),
+                               axisout=1)
 
-
-def _distribute(acq, H):
-    if acq.comm.size > 1:
-        H = H(MPIDistributionIdentityOperator(acq.comm))
-    return H
+#    nbytes = acq.get_projection_nbytes()
+#    if max_nbytes is None or nbytes <= max_nbytes:
+#        if acq.comm.size > 1:
+#            P = P.operands[0]
+#        if isinstance(P, BlockColumnOperator):
+#            for _ in P.operands:
+#                _.restrict(mask)
+#        else:
+#            P.restrict(mask)
+#        return P
+#    n = int(np.ceil(nbytes / max_nbytes))
+#    samplings = acq.sampling.split(n)
+#    acqs = [QubicAcquisition(acq.instrument, _, acq.scene, acq.block)
+#            for _ in samplings]
+#
+#    def callback(i):
+#        p = acqs[i].get_projection_operator(verbose=False)
+#        p.restrict(mask)
+#        return p
+#    return BlockColumnOperator(proxy_group(n, callback), axisout=1)
 
 
 def map2tod(acq, map, convolution=False, max_nbytes=None):
@@ -113,7 +79,7 @@ def map2tod(acq, map, convolution=False, max_nbytes=None):
         convolution = acq.get_convolution_peak_operator()
         map = convolution(map)
 
-    H = _get_operator(acq, max_nbytes=max_nbytes)
+    H = acq.get_operator()
     tod = H(map)
 
     if convolution:
@@ -124,8 +90,10 @@ def map2tod(acq, map, convolution=False, max_nbytes=None):
 
 def _tod2map(acq, tod, coverage_threshold, disp_pcg, disp_pmatrix,
              max_nbytes, tol):
-    projection = _get_projection(acq, max_nbytes=max_nbytes,
-                                 verbose=disp_pmatrix)
+    projection = acq.get_projection_operator(verbose=disp_pmatrix)
+    distribution = acq.get_distribution_operator()
+    P = projection * distribution
+
     shape = len(acq.instrument), len(acq.sampling)
     kind = acq.scene.kind
     if kind == 'I':
@@ -135,17 +103,16 @@ def _tod2map(acq, tod, coverage_threshold, disp_pcg, disp_pmatrix,
         ones[..., 0] = 1
     else:
         raise NotImplementedError()
-    coverage = projection.T(ones)
+    coverage = P.T(ones)
     if kind == 'IQU':
         coverage = coverage[..., 0]
     mask = coverage > coverage_threshold
-    projection = _get_projection_restricted(
-        acq, projection, mask, max_nbytes=max_nbytes, verbose=disp_pmatrix)
+    projection = _get_projection_restricted(acq, projection, mask)
     pack = PackOperator(mask, broadcast='rightward')
-    hwp = _get_hwp(acq, max_nbytes=max_nbytes)
+    hwp = acq.get_hwp_operator()
     polarizer = acq.get_polarizer_operator()
     response = acq.get_detector_response_operator()
-    H = response * polarizer * (hwp * projection)
+    H = response * polarizer * (hwp * projection) * distribution
     if acq.scene.kind == 'QU':
         H = acq.get_subtract_grid_operator() * H
 
@@ -233,7 +200,7 @@ def tod2map_each(acquisition, tod, coverage_threshold=0, disp=True, tol=1e-4,
     if disp:
         bar = progress_bar(len(instrument), 'TOD2MAP_EACH')
     for i, t in izip(instrument, tod):
-        acq = QubicAcquisition(i, acquisition.sampling)
+        acq = QubicAcquisition(i, acquisition.sampling, acquisition.scene)
         x_, n_ = _tod2map(acq, t[None, :], coverage_threshold, False, False,
                           max_nbytes, tol)
         x += x_
